class uniform_t : public vector_field_t {
public:
  MatrixXd w;
  uniform_t() { w.resize(2,1); }
  void field(MatrixXd &x,MatrixXd &V) { V = w;  }
};

class segment_t : public shape_t {
public:
  int N;
  MatrixXd start,end;
  segment_t() { start.resize(2,1); end.resize(2,1); }
  int size() { return N; }
  void xnode(int j,MatrixXd &x) {
    x = start+(end-start)*double(j)/N;
  }
};

class circle_t : public shape_t {
public:
  int N;
  double R;
  MatrixXd center;
  circle_t() {
    N = 100;
    R = 1.0;
    center.resize(2,1);
    center << 0,0;
  }
  int size() { return N; }
  void xnode(int j,MatrixXd &x) {
    double phi = 2*M_PI*double(j)/N;
    x.resize(2,1);
    x << 0.5+cos(phi), sin(phi);
  }
};
  
typedef double (*flux_t)(shape_t &shape,vector_field_t &vf);

class radial_t : public vector_field_t {
public:
  MatrixXd center;
  double intensity;
  radial_t() {
    center.resize(2,1);
    center << 0,0;
    intensity = 1;
  }
  void field(MatrixXd &x,MatrixXd &V) {
    V = intensity*(x-center);
  }
};

class solenoidal_t : public vector_field_t {
public:
  double intensity;
  solenoidal_t() { intensity=1.0; }
  void field(MatrixXd &x,MatrixXd &V) {
    V.resize(2,1);
    V(1,0) = intensity*x(0,0);
    V(0,0) = -intensity*x(1,0);
  }
};

void check_flux(flux_t flux_fun) {
  segment_t segment;
  uniform_t uniform;

  segment.N = 100;
  segment.start << 0,0;
  segment.end << 1,0;
  uniform.w << 0,1;
  printf("flux (segment[1,0], uniform(0,1): %f\n",flux_fun(segment,uniform));

  uniform.w << 1,0;
  printf("flux (segment[1,0], uniform(1,0): %f\n",flux_fun(segment,uniform));

  segment.start << 0,0;
  segment.end << 0,1;
  uniform.w << 1,0;
  printf("flux (segment[0,1], uniform(1,0): %f\n",flux_fun(segment,uniform));

  uniform.w << 0,1;
  printf("flux (segment[0,1], uniform(0,1): %f\n",flux_fun(segment,uniform));

  radial_t radial;
  circle_t circle;
  printf("flux (circle(c=(0,0),R=1), radial(c=(0,0),int=1): %f\n",
         flux_fun(circle,radial));
  circle.N = 1000;
  printf("flux circle(c=(0,0),R=1), radial(c=(0,0),intens=1), N=1000: %f\n",
         flux_fun(circle,radial));

  radial.intensity = 10.0;
  printf("flux circle(c=(0,0),R=1), radial(c=(0,0),intens=100): %f\n",
         flux_fun(circle,radial));

  solenoidal_t sole;
  printf("flux circle(c=(0,0),R=1), sole(c=(0,0),intens=100): %f\n",
         flux_fun(circle,sole));
}
