#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cmath>

#include <iostream>
#include <list>
#include <vector>
#include <set>
#include <omp.h>
#include "boost/multi_array.hpp"
#include <Eigen/Dense>

using namespace std;
using namespace boost;
using Eigen::MatrixXd;
using Eigen::MatrixXi;

class graph_t {
public:
  virtual int size()=0;
  virtual const set<int>& adjacency(int vertex)=0;

  void breadth_first(int start,vector<int> &path,
                     vector<int> &distance) {
    list<int> front;
    front.push_back(start);
    int N = size();
    vector<bool> visited(N,false);
    path.resize(N,0);
    distance.resize(N,0);
    int order=0;
    // path[order++] = start;
    distance[start] = 0;
    while (order<N) {
      int v;
      if (!front.empty()) {
        v = front.front();
        front.pop_front();
      } else {
        for (int j=0; j<N; j++)
          if (!visited[j]) { v=j; break; }
        distance[v] = -1;
      }
      visited[v] = true;
      path[order++] = v;
      const set<int>& ngbrs = adjacency(v);
      set<int>::const_iterator q = ngbrs.begin();
      while (q!=ngbrs.end()) {
        if (!visited[*q]) {
          front.push_back(*q);
          visited[*q] = true;
          int d = distance[v];
          distance[*q] = (d<0? d : d+1);
        }
        q++;
      }
    }
    assert(front.empty());
    for (int j=0; j<N; j++) assert(visited[j]);
  }

  int colorize(vector<int> colors) {
    int color=0, N=size();
    colors.clear();
    colors.resize(N,-1);
    set<int> nocolored;
    for (int j=0; j<N; j++) nocolored.insert(j);
    while (!nocolored.empty()) {
      set<int>::iterator q = nocolored.begin();
      while (q!=nocolored.end()) {
        const set<int> &ngbrs = adjacency(*q);
        set<int>::const_iterator w = ngbrs.begin();
        while (w!=ngbrs.end())
          if (colors[*w++]==color) break;
        if (w==ngbrs.end()) {
          nocolored.erase(*q);
          colors[*q] = color;
          // printf("vertex %d, color %d\n",*q,color);
        }
        q++;
      }
      color++;
    }
    return color;
  }
  
  int distance(int v1,int v2) {
    vector<int> path,d;
    breadth_first(v1,path,d);
    return d[v2];
  }
};

class stored_graph_t : public graph_t {
private:
  int N;
  vector< set<int> > adj;
public:
  stored_graph_t(int N_=0) : N(N_) { adj.resize(N); }
  void set_edge(int v1,int v2,bool s) {
    assert(v1>=0);
    assert(v1<N);
    assert(v2>=0);
    assert(v2<N);
    assert(v1!=v2);
    if (s) {
      adj[v1].insert(v2);
      adj[v2].insert(v1);
    } else {
      adj[v1].erase(v2);
      adj[v2].erase(v1);
    }
  }
  void add_edge(int v1,int v2) { set_edge(v1,v2,true); }
  void remove_edge(int v1,int v2) { set_edge(v1,v2,false); }
  int size() { return adj.size(); };
  set<int>& adjacency(int v) { return adj[v]; }
  void resize(int N_) { adj.clear(); adj.resize(N); }
};
